@@ -2,6 +2,7 @@
  * This file provides the low-level bindings around the C++ simdjson library,
  * exposing it to Python as the csimdjson module.
  */
+#define PY_SSIZE_T_CLEAN
 #include <pybind11/pybind11.h>
 #include <Python.h>
 #include "simdjson.h"
@@ -222,12 +223,33 @@ PYBIND11_MODULE(csimdjson, m) {
                     PyErr_SetString(PyExc_IOError, e.what());
                     return;
                 case error_code::UTF8_ERROR:
-                    PyErr_SetString(PyExc_UnicodeDecodeError, e.what());
+                {
+                    // simdjson doesn't yet give us any precise details on
+                    // where the error occured. See upstream#46.
+                    // PyUnicodeDecodeError_* methods are stubs on PyPy3,
+                    // so we can't use that helper..
+                    PyObject *unicode_error = PyObject_CallFunction(
+                        PyExc_UnicodeDecodeError,
+                        "sy#nns",
+                        "utf-8",
+                        "",
+                        0,
+                        0,
+                        1,
+                        e.what()
+                    );
+                    PyErr_SetObject(
+                        PyExc_UnicodeDecodeError,
+                        unicode_error
+                    );
+                    Py_XDECREF(unicode_error);
                     return;
+                }
                 default:
                     PyErr_SetString(PyExc_RuntimeError, e.what());
                     return;
             }
+            throw;
         }
     });
 
